/*
Parrot
Copyright (C) 2003 Douglas Thain
http://www.cs.wisc.edu/~thain/research/parrot
This program is released under the GNU General Public License.
See the file COPYING for details.
*/

#ifndef PFS_PROCESS_H
#define PFS_PROCESS_H

#include "pfs_types.h"
#include "pfs_table.h"
#include "pfs_sysdeps.h"

extern "C" {
#include "tracer.h"
}

#include <sys/types.h>
#include <sys/resource.h>
#include <sys/time.h>

#define PFS_PROCESS_FLAGS_STARTUP 1
#define PFS_PROCESS_FLAGS_ASYNC 2

#define PFS_PROCESS_STATE_KERNEL 0
#define PFS_PROCESS_STATE_USER 1
#define PFS_PROCESS_STATE_WAITPID 2
#define PFS_PROCESS_STATE_WAITREAD 3
#define PFS_PROCESS_STATE_WAITWRITE 4
#define PFS_PROCESS_STATE_DONE 5


struct pfs_process {
	char name[PFS_PATH_MAX];
	char new_logical_name[PFS_PATH_MAX];
	char new_physical_name[PFS_PATH_MAX];
	char scratch_data[PFS_SCRATCH_SIZE];

	mode_t umask;
	pid_t pid, ppid;
	int flags, state;
	pfs_table *table;
	struct tracer *tracer;
	struct timeval seltime;

	pfs_size_t io_channel_offset;

	int syscall;
	int syscall_dummy;
	int syscall_result;
	int syscall_args[TRACER_ARGS_MAX];
	int syscall_args_changed;
	int actual_result;
	int did_stream_warning;

	pid_t          wait_pid;
	int           *wait_ustatus;
	struct rusage *wait_urusage;
	int            wait_options;

	struct rusage  exit_rusage;
	int            exit_status;
	int	       exit_signal;
	int            interrupted;
};

struct pfs_process * pfs_process_create( pid_t pid, pid_t ppid, int share_table, int exit_signal );
struct pfs_process * pfs_process_lookup( pid_t pid );
void pfs_process_delete( struct pfs_process *p );

void pfs_process_stop( struct pfs_process *p, int status, struct rusage usage );
int pfs_process_waitpid( struct pfs_process *p, pid_t wait_pid, int *wait_ustatus, int wait_options, struct rusage *wait_urusage );

void pfs_process_sigio();
void pfs_process_wake( pid_t pid );
int  pfs_process_count();
int  pfs_process_raise( pid_t pid, int sig );

extern "C" int pfs_process_getpid();
extern "C" char * pfs_process_name();
extern "C" void pfs_process_kill();
extern "C" void pfs_process_killall();

extern struct pfs_process *current;

#endif
